"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.interpolateArgsIntoCommand = exports.LARGE_BUFFER = void 0;
const child_process_1 = require("child_process");
const path = require("path");
const yargsParser = require("yargs-parser");
const npm_run_path_1 = require("npm-run-path");
const chalk = require("chalk");
const pseudo_terminal_1 = require("../../tasks-runner/pseudo-terminal");
exports.LARGE_BUFFER = 1024 * 1000000;
async function loadEnvVars(path) {
    if (path) {
        const result = (await Promise.resolve().then(() => require('dotenv'))).config({ path });
        if (result.error) {
            throw result.error;
        }
    }
    else {
        try {
            (await Promise.resolve().then(() => require('dotenv'))).config();
        }
        catch { }
    }
}
const propKeys = [
    'command',
    'commands',
    'color',
    'parallel',
    'readyWhen',
    'cwd',
    'args',
    'envFile',
    '__unparsed__',
    'env',
    'usePty',
    'streamOutput',
    'verbose',
];
async function default_1(options, context) {
    await loadEnvVars(options.envFile);
    const normalized = normalizeOptions(options);
    if (options.readyWhen && !options.parallel) {
        throw new Error('ERROR: Bad executor config for run-commands - "readyWhen" can only be used when "parallel=true".');
    }
    if (options.commands.find((c) => c.prefix || c.color || c.bgColor) &&
        !options.parallel) {
        throw new Error('ERROR: Bad executor config for run-commands - "prefix", "color" and "bgColor" can only be set when "parallel=true".');
    }
    try {
        const result = options.parallel
            ? await runInParallel(normalized, context)
            : await runSerially(normalized, context);
        return result;
    }
    catch (e) {
        if (process.env.NX_VERBOSE_LOGGING === 'true') {
            console.error(e);
        }
        throw new Error(`ERROR: Something went wrong in run-commands - ${e.message}`);
    }
}
exports.default = default_1;
async function runInParallel(options, context) {
    const procs = options.commands.map((c) => createProcess(null, c, options.readyWhen, options.color, calculateCwd(options.cwd, context), options.env ?? {}, true, options.usePty, options.streamOutput).then((result) => ({
        result,
        command: c.command,
    })));
    let terminalOutput = '';
    if (options.readyWhen) {
        const r = await Promise.race(procs);
        terminalOutput += r.result.terminalOutput;
        if (!r.result.success) {
            const output = `Warning: command "${r.command}" exited with non-zero status code`;
            terminalOutput += output;
            if (options.streamOutput) {
                process.stderr.write(output);
            }
            return { success: false, terminalOutput };
        }
        else {
            return { success: true, terminalOutput };
        }
    }
    else {
        const r = await Promise.all(procs);
        terminalOutput += r.map((f) => f.result.terminalOutput).join('');
        const failed = r.filter((v) => !v.result.success);
        if (failed.length > 0) {
            const output = failed
                .map((f) => `Warning: command "${f.command}" exited with non-zero status code`)
                .join('\r\n');
            terminalOutput += output;
            if (options.streamOutput) {
                process.stderr.write(output);
            }
            return {
                success: false,
                terminalOutput,
            };
        }
        else {
            return {
                success: true,
                terminalOutput,
            };
        }
    }
}
function normalizeOptions(options) {
    if (options.command) {
        options.commands = [{ command: options.command }];
        options.parallel = !!options.readyWhen;
    }
    else {
        options.commands = options.commands.map((c) => typeof c === 'string' ? { command: c } : c);
    }
    if (options.args && Array.isArray(options.args)) {
        options.args = options.args.join(' ');
    }
    const unparsedCommandArgs = yargsParser(options.__unparsed__, {
        configuration: {
            'parse-numbers': false,
            'parse-positional-numbers': false,
            'dot-notation': false,
        },
    });
    options.unknownOptions = Object.keys(options)
        .filter((p) => propKeys.indexOf(p) === -1 && unparsedCommandArgs[p] === undefined)
        .reduce((m, c) => ((m[c] = options[c]), m), {});
    options.parsedArgs = parseArgs(unparsedCommandArgs, options.unknownOptions, options.args);
    options.commands.forEach((c) => {
        c.command = interpolateArgsIntoCommand(c.command, options, c.forwardAllArgs ?? true);
    });
    return options;
}
async function runSerially(options, context) {
    const pseudoTerminal = pseudo_terminal_1.PseudoTerminal.isSupported()
        ? (0, pseudo_terminal_1.getPseudoTerminal)()
        : null;
    let terminalOutput = '';
    for (const c of options.commands) {
        const result = await createProcess(pseudoTerminal, c, undefined, options.color, calculateCwd(options.cwd, context), options.env ?? {}, false, options.usePty, options.streamOutput);
        terminalOutput += result.terminalOutput;
        if (!result.success) {
            const output = `Warning: command "${c.command}" exited with non-zero status code`;
            result.terminalOutput += output;
            if (options.streamOutput) {
                process.stderr.write(output);
            }
            return { success: false, terminalOutput };
        }
    }
    return { success: true, terminalOutput };
}
async function createProcess(pseudoTerminal, commandConfig, readyWhen, color, cwd, env, isParallel, usePty = true, streamOutput = true) {
    env = processEnv(color, cwd, env);
    // The rust runCommand is always a tty, so it will not look nice in parallel and if we need prefixes
    // currently does not work properly in windows
    if (pseudoTerminal &&
        process.env.NX_NATIVE_COMMAND_RUNNER !== 'false' &&
        !commandConfig.prefix &&
        !isParallel &&
        usePty) {
        const cp = pseudoTerminal.runCommand(commandConfig.command, {
            cwd,
            jsEnv: env,
            quiet: !streamOutput,
        });
        let terminalOutput = '';
        return new Promise((res) => {
            cp.onOutput((output) => {
                terminalOutput += output;
                if (readyWhen && output.indexOf(readyWhen) > -1) {
                    res({ success: true, terminalOutput });
                }
            });
            cp.onExit((code) => {
                if (code >= 128) {
                    process.exit(code);
                }
                else {
                    res({ success: code === 0, terminalOutput });
                }
            });
        });
    }
    return nodeProcess(commandConfig, cwd, env, readyWhen, streamOutput);
}
function nodeProcess(commandConfig, cwd, env, readyWhen, streamOutput = true) {
    let terminalOutput = '';
    return new Promise((res) => {
        const childProcess = (0, child_process_1.exec)(commandConfig.command, {
            maxBuffer: exports.LARGE_BUFFER,
            env,
            cwd,
        });
        /**
         * Ensure the child process is killed when the parent exits
         */
        const processExitListener = (signal) => childProcess.kill(signal);
        process.on('exit', processExitListener);
        process.on('SIGTERM', processExitListener);
        process.on('SIGINT', processExitListener);
        process.on('SIGQUIT', processExitListener);
        childProcess.stdout.on('data', (data) => {
            const output = addColorAndPrefix(data, commandConfig);
            terminalOutput += output;
            if (streamOutput) {
                process.stdout.write(output);
            }
            if (readyWhen && data.toString().indexOf(readyWhen) > -1) {
                res({ success: true, terminalOutput });
            }
        });
        childProcess.stderr.on('data', (err) => {
            const output = addColorAndPrefix(err, commandConfig);
            terminalOutput += output;
            if (streamOutput) {
                process.stderr.write(output);
            }
            if (readyWhen && err.toString().indexOf(readyWhen) > -1) {
                res({ success: true, terminalOutput });
            }
        });
        childProcess.on('error', (err) => {
            const ouptput = addColorAndPrefix(err.toString(), commandConfig);
            terminalOutput += ouptput;
            if (streamOutput) {
                process.stderr.write(ouptput);
            }
            res({ success: false, terminalOutput });
        });
        childProcess.on('exit', (code) => {
            if (!readyWhen) {
                res({ success: code === 0, terminalOutput });
            }
        });
    });
}
function addColorAndPrefix(out, config) {
    if (config.prefix) {
        out = out
            .split('\n')
            .map((l) => l.trim().length > 0 ? `${chalk.bold(config.prefix)} ${l}` : l)
            .join('\n');
    }
    if (config.color && chalk[config.color]) {
        out = chalk[config.color](out);
    }
    if (config.bgColor && chalk[config.bgColor]) {
        out = chalk[config.bgColor](out);
    }
    return out;
}
function calculateCwd(cwd, context) {
    if (!cwd)
        return context.root;
    if (path.isAbsolute(cwd))
        return cwd;
    return path.join(context.root, cwd);
}
function processEnv(color, cwd, env) {
    const localEnv = (0, npm_run_path_1.env)({ cwd: cwd ?? process.cwd() });
    const res = {
        ...process.env,
        ...localEnv,
        ...env,
    };
    // need to override PATH to make sure we are using the local node_modules
    if (localEnv.PATH)
        res.PATH = localEnv.PATH; // UNIX-like
    if (localEnv.Path)
        res.Path = localEnv.Path; // Windows
    if (color) {
        res.FORCE_COLOR = `${color}`;
    }
    return res;
}
function interpolateArgsIntoCommand(command, opts, forwardAllArgs) {
    if (command.indexOf('{args.') > -1) {
        const regex = /{args\.([^}]+)}/g;
        return command.replace(regex, (_, group) => opts.parsedArgs[group] !== undefined ? opts.parsedArgs[group] : '');
    }
    else if (forwardAllArgs) {
        let args = '';
        if (Object.keys(opts.unknownOptions ?? {}).length > 0) {
            args +=
                ' ' +
                    Object.keys(opts.unknownOptions)
                        .filter((k) => typeof opts.unknownOptions[k] !== 'object' &&
                        opts.parsedArgs[k] === opts.unknownOptions[k])
                        .map((k) => `--${k} ${opts.unknownOptions[k]}`)
                        .join(' ');
        }
        if (opts.args) {
            args += ` ${opts.args}`;
        }
        if (opts.__unparsed__?.length > 0) {
            args += ` ${opts.__unparsed__.join(' ')}`;
        }
        return `${command}${args}`;
    }
    else {
        return command;
    }
}
exports.interpolateArgsIntoCommand = interpolateArgsIntoCommand;
function parseArgs(unparsedCommandArgs, unknownOptions, args) {
    if (!args) {
        return { ...unknownOptions, ...unparsedCommandArgs };
    }
    return yargsParser(args.replace(/(^"|"$)/g, ''), {
        configuration: { 'camel-case-expansion': false },
    });
}
